//
//  MANDELBROT v1
//
//  Mandelbrot set generator for the TI-89 calculator
//
//  (c) 2002 by Stephan Walter
//  Released under the Beer-ware License
//
//  "THE BEER-WARE LICENSE" (Revision 42):
//  <stephan.walter@gmx.ch> wrote this file.   As long as you retain this
//  notice you can do whatever you want with this stuff.  If we meet some
//  day,  and you think this stuff is worth it,  you can buy me a beer in
//  return.                                                Stephan Walter
//

#define USE_TI89		// Programm luft nur auf einem TI-89

#define OPTIMIZE_ROM_CALLS	// etwas schneller
#define SAVE_SCREEN		// Bildschirminhalt speichern und wiederherstellen
#define NO_EXIT_SUPPORT
#define NO_CALC_DETECT

#include <tigcclib.h>

/*	SCALE = 1 << SCALE_BITS
	kann nicht direkt eingesetzt werden
	Bei grsseren Werten kommt es zu Fehlern  */
#define SCALE_BITS 26
#define SCALE 67108864


void draw_mandel_int(float fcre_start, float fcre_end, float fcim_start, float fcim_end, short maxit)
{
	short i,y,x;
	long zre,zim,cre,cre1,cim,cim1,zre_neu; //,zim_neu;

	long cre_start = fcre_start * SCALE;
	long cre_end = fcre_end * SCALE;
	long cim_start = fcim_start * SCALE;
	long cim_end = fcim_end * SCALE;

	long dcre = (cre_end-cre_start) / 160;
	long dcim = (cim_end-cim_start) / 100;

	cim1 = cim_start;

	for(y=0;y<100;y++) {
		cre = cre1 = cre_start;
		for(x=0;x<160;x++) {
			cim = cim1;
			GraySetAMSPlane(LIGHT_PLANE);
			DrawPix(x,y,A_NORMAL);
			GraySetAMSPlane(DARK_PLANE);
			DrawPix(x,y,A_NORMAL);

			zre = zim = 0;
			for(i=0;i<maxit;i++)
			{	/*	zre := zre^2 - zim^2 = (zre-zim) * (zre+zim)
					zim := 2 * zre * zim	*/
				zre_neu = ((long long)(zre-zim) * (long long)(zre+zim))>>SCALE_BITS;
				zim = (((long long)zre*(long long)zim)>>(SCALE_BITS-1)) + cim;
				zre = zre_neu + cre;
				if((abs(zre)>2*SCALE) && (abs(zim)>2*SCALE)) {
					if(i>=5)
						DrawPix(x,y,A_REVERSE);
					if(i>10 || (i<5 && i>2)) {
						GraySetAMSPlane(LIGHT_PLANE);
						DrawPix(x,y,A_REVERSE);
					}
					break;
				}
			}
			cre = cre1 += dcre;
		}
		cim1 += dcim;
		//if(kbhit()) return;
	}
}


void _main(void)
{
	char siter[20]="50";
	short taste=0;
	short flag=2;
	short maxit;
	float cre_start, cre_end, cim_start, cim_end;

	HANDLE handle = DialogNewSimple (125, 60);

	DialogAddTitle (handle, "MANDELBROT v1", BT_OK, BT_CANCEL);
	DialogAddText (handle, 20, 14, "(c) 2002 Stephan Walter");
	DialogAddText (handle, 17, 21, "http://cowww.epfl.ch/~swalter/");
	DialogAddRequest (handle, 15, 35, "max. Iterationen", 0, 4, 4);

	while (DialogDo (handle, CENTER, CENTER, siter, NULL) == KEY_ENTER) {
		clrscr();

		maxit=atoi(siter);
		GrayOn();

		cre_start = -2;
		cre_end = 1.2;
		cim_start = 1;
		cim_end = -1;

		do {
			//taste = ngetchx();
			DrawClipRect (&(WIN_RECT){(cre_start+2)*50, (cim_start+1)*80, (cre_end+2)*50, (cim_end+1)*80}, &(SCR_RECT){{0, 0, 159, 99}}, A_XOR);
			switch (flag) {
				case 2:
					flag = 0;
					draw_mandel_int(cre_start, cre_end, cim_start, cim_end, maxit);
					break;
				case 0:
					switch (taste) {
						case 337: //UP
							cim_start = min(1,cim_start+0.1);
							break;
						case 340: //DOWN
							cim_start = max(cim_end, cim_start-0.1);
							break;
						case 338: //LEFT
							cre_start = max(-2,cre_start-0.1);
							break;
						case 344: //RIGHT
							cre_start = min(cre_end,cre_start+0.1);
					}
					break;
				case 1:
					switch (taste) {
						case 340: //UP
							cim_end = min(cim_start,cim_end+0.1);
							break;
						case 337: //DOWN
							cim_end = max(-1, cim_end-0.1);
							break;
						case 338: //LEFT
							cre_end = max(cre_start,cre_end-0.1);
							break;
						case 344: //RIGHT
							cre_end = min(1.2,cre_end+0.1);
					}
					break;
			}
			DrawClipRect (&(WIN_RECT){(cre_start+2)*50, (cim_start+1)*80, (cre_end+2)*50, (cim_end+1)*80}, &(SCR_RECT){{0, 0, 159, 99}}, A_XOR);

			if (taste == KEY_ENTER)
				flag +=1;

			taste = ngetchx();

		} while (taste != KEY_ESC);
		GrayOff();
	}
}
